@@ -19,33 +19,33 @@ namespace Fortran::semantics {
 void DeallocateChecker::Leave(const parser::DeallocateStmt &deallocateStmt) {
   for (const parser::AllocateObject &allocateObject :
       std::get<std::list<parser::AllocateObject>>(deallocateStmt.t)) {
-    parser::CharBlock source;
-    const Symbol *symbol{nullptr};
     common::visit(
         common::visitors{
             [&](const parser::Name &name) {
-              source = name.source;
-              symbol = name.symbol;
+              const Symbol *symbol{
+                  name.symbol ? &name.symbol->GetUltimate() : nullptr};
+              ;
               if (context_.HasError(symbol)) {
                 // already reported an error
               } else if (!IsVariableName(*symbol)) {
                 context_.Say(name.source,
                     "Name in DEALLOCATE statement must be a variable name"_err_en_US);
-              } else if (!IsAllocatableOrPointer(
-                             symbol->GetUltimate())) { // C932
+              } else if (!IsAllocatableOrObjectPointer(symbol)) { // C936
                 context_.Say(name.source,
                     "Name in DEALLOCATE statement must have the ALLOCATABLE or POINTER attribute"_err_en_US);
               } else if (auto whyNot{WhyNotDefinable(name.source,
                              context_.FindScope(name.source),
                              {DefinabilityFlag::PointerDefinition,
-                                 DefinabilityFlag::AcceptAllocatable},
+                                 DefinabilityFlag::AcceptAllocatable,
+                                 DefinabilityFlag::PotentialDeallocation},
                              *symbol)}) {
                 // Catch problems with non-definability of the
                 // pointer/allocatable
                 context_
                     .Say(name.source,
                         "Name in DEALLOCATE statement is not definable"_err_en_US)
-                    .Attach(std::move(*whyNot));
+                    .Attach(std::move(
+                        whyNot->set_severity(parser::Severity::Because)));
               } else if (auto whyNot{WhyNotDefinable(name.source,
                              context_.FindScope(name.source),
                              DefinabilityFlags{}, *symbol)}) {
@@ -53,7 +53,8 @@ void DeallocateChecker::Leave(const parser::DeallocateStmt &deallocateStmt) {
                 context_
                     .Say(name.source,
                         "Object in DEALLOCATE statement is not deallocatable"_err_en_US)
-                    .Attach(std::move(*whyNot));
+                    .Attach(std::move(
+                        whyNot->set_severity(parser::Severity::Because)));
               } else {
                 context_.CheckIndexVarRedefine(name);
               }
@@ -61,30 +62,38 @@ void DeallocateChecker::Leave(const parser::DeallocateStmt &deallocateStmt) {
             [&](const parser::StructureComponent &structureComponent) {
               // Only perform structureComponent checks if it was successfully
               // analyzed by expression analysis.
-              source = structureComponent.component.source;
-              symbol = structureComponent.component.symbol;
+              auto source{structureComponent.component.source};
               if (const auto *expr{GetExpr(context_, allocateObject)}) {
-                if (symbol) {
-                  if (!IsAllocatableOrPointer(*symbol)) { // C932
-                    context_.Say(source,
-                        "Component in DEALLOCATE statement must have the ALLOCATABLE or POINTER attribute"_err_en_US);
-                  } else if (auto whyNot{WhyNotDefinable(source,
-                                 context_.FindScope(source),
-                                 {DefinabilityFlag::PointerDefinition,
-                                     DefinabilityFlag::AcceptAllocatable},
-                                 *expr)}) {
-                    context_
-                        .Say(source,
-                            "Name in DEALLOCATE statement is not definable"_err_en_US)
-                        .Attach(std::move(*whyNot));
-                  } else if (auto whyNot{WhyNotDefinable(source,
-                                 context_.FindScope(source),
-                                 DefinabilityFlags{}, *expr)}) {
-                    context_
-                        .Say(source,
-                            "Object in DEALLOCATE statement is not deallocatable"_err_en_US)
-                        .Attach(std::move(*whyNot));
-                  }
+                if (const Symbol *
+                        symbol{structureComponent.component.symbol
+                                ? &structureComponent.component.symbol
+                                       ->GetUltimate()
+                                : nullptr};
+                    !IsAllocatableOrObjectPointer(symbol)) { // F'2023 C936
+                  context_.Say(source,
+                      "Component in DEALLOCATE statement must have the ALLOCATABLE or POINTER attribute"_err_en_US);
+                } else if (auto whyNot{WhyNotDefinable(source,
+                               context_.FindScope(source),
+                               {DefinabilityFlag::PointerDefinition,
+                                   DefinabilityFlag::AcceptAllocatable,
+                                   DefinabilityFlag::PotentialDeallocation},
+                               *expr)}) {
+                  context_
+                      .Say(source,
+                          "Name in DEALLOCATE statement is not definable"_err_en_US)
+                      .Attach(std::move(
+                          whyNot->set_severity(parser::Severity::Because)));
+                } else if (auto whyNot{WhyNotDefinable(source,
+                               context_.FindScope(source), DefinabilityFlags{},
+                               *expr)}) {
+                  context_
+                      .Say(source,
+                          "Object in DEALLOCATE statement is not deallocatable"_err_en_US)
+                      .Attach(std::move(
+                          whyNot->set_severity(parser::Severity::Because)));
+                } else if (evaluate::ExtractCoarrayRef(*expr)) { // F'2023 C955
+                  context_.Say(source,
+                      "Component in DEALLOCATE statement may not be coindexed"_err_en_US);
                 }
               }
             },
